/**
 * Using Rails-like standard naming convention for endpoints.
 * GET     /api/comments              ->  index
 * POST    /api/comments              ->  create
 * GET     /api/comments/:id          ->  show
 * PUT     /api/comments/:id          ->  update
 * DELETE  /api/comments/:id          ->  destroy
 */

'use strict';

var _ = require('lodash');
var Comment = require('./comment.model');
var User = require('../user/user.model');

function handleError(res, statusCode) {
  statusCode = statusCode || 500;
  return function(err) {
    res.status(statusCode).send(err);
  };
}

function responseWithResult(res, dig, statusCode) {
  statusCode = statusCode || 200;
  var promise = [];
  return function(entity) {
    if (entity) {
      if (dig) {
        entity.forEach(function(obj, index) {
          promise.push(new Promise(function(resolve, reject) {
            User.findById(obj.setterId, function(err, userObj) {
              obj.user = userObj;
              resolve(obj);
            });
          }));
        });
        Promise.all(promise).then(function(data) {
          console.log(data);
          res.status(statusCode).json(data);
        });
      } else {
        User.findById(entity.userId, function(err, userObj) {
          entity.user = userObj;
          res.status(statusCode).json(entity);
        });
      }
    }
  };
}

function handleEntityNotFound(res) {
  return function(entity) {
    if (!entity) {
      res.status(404).end();
      return null;
    }
    return entity;
  };
}

function saveUpdates(updates) {
  return function(entity) {
    var updated = _.merge(entity, updates);
    return updated.saveAsync()
      .spread(function(updated) {
        return updated;
      });
  };
}

function removeEntity(res) {
  return function(entity) {
    if (entity) {
      return entity.removeAsync()
        .then(function() {
          res.status(204).end();
        });
    }
  };
}

// Gets a list of Comments
exports.index = function(req, res) {
  var limit = req.query.limit;
  var userId = req.query.user;
  var active = req.query.active;
  var sq = {};
  if (userId) {
    sq.getterId = userId
  }
  if (active) {
    sq.active = active
  }
  Comment.find(sq).sort( { dateAdded: -1 } ).lean().limit(limit)
    .then(responseWithResult(res, true))
    .catch(handleError(res));
};

// Gets a single Comment from the DB
exports.show = function(req, res) {
  Comment.findByIdAsync(req.params.id)
    .then(handleEntityNotFound(res))
    .then(responseWithResult(res))
    .catch(handleError(res));
};

// Creates a new Comment in the DB
exports.create = function(req, res) {
  Comment.createAsync(req.body)
    .then(responseWithResult(res, 201))
    .catch(handleError(res));
};

// Updates an existing Comment in the DB
exports.update = function(req, res) {
  if (req.body._id) {
    delete req.body._id;
  }
  Comment.findByIdAsync(req.params.id)
    .then(handleEntityNotFound(res))
    .then(saveUpdates(req.body))
    .then(responseWithResult(res))
    .catch(handleError(res));
};

// Deletes a Comment from the DB
exports.destroy = function(req, res) {
  Comment.findByIdAsync(req.params.id)
    .then(handleEntityNotFound(res))
    .then(removeEntity(res))
    .catch(handleError(res));
};
